import java.util.Hashtable;

import java.net.MalformedURLException;
import java.net.UnknownServiceException;
import java.net.URL;

import net.jxta.discovery.DiscoveryService;

import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredDocument;

import net.jxta.exception.PeerGroupException;

import net.jxta.id.IDFactory;

import net.jxta.impl.peergroup.StdPeerGroupParamAdv;

import net.jxta.impl.protocol.EndpointAdv;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.peergroup.PeerGroupFactory;

import net.jxta.platform.ModuleClassID;
import net.jxta.platform.ModuleSpecID;

import net.jxta.protocol.ModuleClassAdvertisement;
import net.jxta.protocol.ModuleImplAdvertisement;
import net.jxta.protocol.ModuleSpecAdvertisement;
import net.jxta.protocol.PeerGroupAdvertisement;


/**
 * La classe principale responsable de la cration du groupe de pair "net send"
 * et du dmarrage du service NetSend
 */
public class NetSend
{
    /**     
     * L'ID du groupe de pair pour le groupe NetSend.
     */
    private static final String refPeerGroupID = 
        "urn:jxta:uuid-68B8A7A691684F9C9E05971D66D78ED602";

    /**
     * L'ID de la spcification de module pour l'annonce d'implmentation de
     * module du groupe de pair.
     */
    private static final String refPeerGroupSpec = 
        "urn:jxta:uuid-DEADBEEFDEAFBABAFEEDBABE000000019E2"
        + "DDBBCE5FE4957A139B0ECE8DEB46D06";

    /**    
     * Le nouveau groupe cr par l'application.
     */
    private PeerGroup newGroup = null;
    
    /**
     * Le gourpe "Net Peer Group" pour l'application.     
     */
    private PeerGroup netPeerGroup = null;

    /**    
     * Cre la classe principale de l'application.
     */
    public NetSend()
    {
        super();
    }

    /**     
     * Cre l'annonce de classe de module selon les paramtres donns.
     *    
     * @param       moduleClassID l'ID de classe de module pour l'annonce.
     * @param       name nom symbolique de l'annonce.
     * @param       description la description de l'annonce.
     * @exception   UnknownServiceException si la chane moduleClassID est
     *              mal forme.
     * @exception   MalformedURLException si la chane moduleClassID est
     *              mal forme.
     */
    private ModuleClassAdvertisement createModuleClassAdv(
        String moduleClassID, String name, String description) 
            throws UnknownServiceException, MalformedURLException
    {        
        // Cre l'ID de classe  partir de la chane refModuleClassID.
        ModuleClassID classID = (ModuleClassID) IDFactory.fromURL(
            new URL((moduleClassID)));
        
        // Cre l'annonce de classe du module.
        ModuleClassAdvertisement moduleClassAdv = 
            (ModuleClassAdvertisement) AdvertisementFactory.newAdvertisement(
                ModuleClassAdvertisement.getAdvertisementType());
        
        // Configure l'annonce de classe du module.
        moduleClassAdv.setDescription(description);
        moduleClassAdv.setModuleClassID(classID);
        moduleClassAdv.setName(name);
        
        // Renvoie l'annonce  l'appelant.
        return moduleClassAdv;
    }

    /**
     * Cre l'annonce d'implmentation de module pour le service, en utilisant
     * l'ID de spcification qui est pass dans l'annonce
     * ModuleImplAdvertisement. Utilise le ModuleImplAdvertisement pass pour
     * crer l'lment compatible de la spcification d'implmentation du
     * module.
     *
     * @param       groupImpl le ModuleImplAdvertisement du groupe de pair
     *              parent.     
     * @param       moduleSpecAdv le source de l'ID de spcification.
     * @param       description de l'implmentation de module.
     * @param       code le nom qualifi de la classe de l'implmentation de
     *              module.
     * @return      the l'annonce de l'implmentation de module gnre.
     */
    private ModuleImplAdvertisement createModuleImplAdv(
        ModuleImplAdvertisement groupImpl, 
        ModuleSpecAdvertisement moduleSpecAdv, 
        String description, String code)
    {        
        // Rcupre l'ID de spcification  partir de l'annoce passe.
        ModuleSpecID specID = moduleSpecAdv.getModuleSpecID();
        
        // Cre l'annonce de l'implmentation de module.
        ModuleImplAdvertisement moduleImplAdv = 
           (ModuleImplAdvertisement) AdvertisementFactory.newAdvertisement(
               ModuleImplAdvertisement.getAdvertisementType());

        // Configure l'annonce de l'implmentation de module.
        moduleImplAdv.setCode(code);
        moduleImplAdv.setCompat(groupImpl.getCompat());
        moduleImplAdv.setDescription(description);
        moduleImplAdv.setModuleSpecID(specID);
        moduleImplAdv.setProvider("Bertrand Goetzmann");
        
        // Renvoie de l'annonce  l'appelant.
        return moduleImplAdv;
    }

    /**     
     * Cre une annonce de spcification de module en utilisant les paramtres
     * passs.
     *
     * @param       moduleSpecID l'ID de la spcification de module pour
     *              l'annonce.
     * @param       name nom symbolique de l'annonce.
     * @param       description la description de l'annonce.
     * @exception   UnknownServiceException si la chane moduleSpecID est mal
     *              forme.
     * @exception   MalformedURLException si la chane moduleSpecID est mal
     *              forme.
     */
    private ModuleSpecAdvertisement createModuleSpecAdv(String moduleSpecID, 
        String name, String description) 
            throws UnknownServiceException, MalformedURLException
    {
        // Crer l'ID de spcification  partir de la chane refModuleSpecID.
        ModuleSpecID specID = (ModuleSpecID) IDFactory.fromURL(
            new URL((moduleSpecID)));

        // Cr l'annonce de spcification de module.
        ModuleSpecAdvertisement moduleSpecAdv = 
            (ModuleSpecAdvertisement) AdvertisementFactory.newAdvertisement(
                ModuleSpecAdvertisement.getAdvertisementType());

        // Configure l'annonce de spcification de module.
        moduleSpecAdv.setCreator("Bertrand Goetzmann");
        moduleSpecAdv.setModuleSpecID(specID);
        moduleSpecAdv.setDescription(description);
        moduleSpecAdv.setName(name);
        moduleSpecAdv.setSpecURI(
            "http://www.object-everywhere.com/projects/jxta");
        moduleSpecAdv.setVersion("1.0");

        // Renvoie de l'annonce  l'appelant.
        return moduleSpecAdv;
    }

    /**
     * Cre un groupe de pair et configure l'implmentation de l'application.    
     *
     * @exception   Exception, PeerGroupException est lance si il y a un
     *              un problme lors de la cration du groupe de pair ou de
     *              l'annonce de l'application.
     */    
    public void createPeerGroup() throws Exception, PeerGroupException
    {
        // Le nom et la description du groupe de pair.
        String name = "Group Net Send";
        String description = "Groupe de pair de l'application NetSend";           

        // Le service Discovery  utiliser pour publier les annonces du module
        // et celle du groupe.
        DiscoveryService discovery = netPeerGroup.getDiscoveryService();
        
        ModuleImplAdvertisement implAdv =
            netPeerGroup.getAllPurposePeerGroupImplAdvertisement();               
        
        // Cre les annonces du module pour l'application NetSend
        ModuleClassAdvertisement appClassAdv = createModuleClassAdv(
            NetSendApp.refModuleClassID , "Application NetSend", 
            "Une application permettant d'envoyer des messages.");
        ModuleSpecAdvertisement appSpecAdv = createModuleSpecAdv(
            NetSendApp.refModuleSpecID, "Application NetSend", 
            "La spcification de l'application NetSend");
        ModuleImplAdvertisement appImplAdv = createModuleImplAdv(
            implAdv, appSpecAdv ,
            "L'implmentation de rfrence de l'application NetSend",
            "NetSendApp");        
        
        // Rcupre les paramtres pour l'annonce de l'implmentation de module
        // du groupe de pair pour ajouter notre service.
        StdPeerGroupParamAdv params = 
            new StdPeerGroupParamAdv(implAdv.getParam());       
        
        // Remplacer les applications dans les paramtres.
        Hashtable applications = new Hashtable();

        // Ajouter l'application NetSend.
        applications.put(appClassAdv.getModuleClassID(), appImplAdv);

        // Dfinir les applications dans les paramtres.
        params.setApps(applications);        
                        
        // Dfinit les paramtres sur l'annonce de l'implmentation.
        implAdv.setParam((StructuredDocument) params.getDocument(
            new MimeMediaType("text", "xml")));
        
        // Il faut changer l'ID de la spcifaction du module pour l'annonce de
        // l'implmentation.
        implAdv.setModuleSpecID((ModuleSpecID) IDFactory.fromURL(
            new URL(refPeerGroupSpec)));
               
        // Publier les annonces de classe et de spcification de module.
        discovery.publish(appClassAdv, DiscoveryService.ADV);
        discovery.remotePublish(appClassAdv, DiscoveryService.ADV);
        discovery.publish(appSpecAdv, DiscoveryService.ADV);
        discovery.remotePublish(appSpecAdv, DiscoveryService.ADV);       
        
        // Publier l'annonce d'implmentation du groupe de pair.
        discovery.publish(implAdv, DiscoveryService.ADV);
        discovery.remotePublish(implAdv, DiscoveryService.ADV);
                
        // Crer l'id du groupe de pair.
        PeerGroupID groupID = (PeerGroupID) IDFactory.fromURL(
            new URL((refPeerGroupID)));
        
        // Cre le nouveau goupe en utilisant l'ID de groupe, l'annonce, le nom
        // et la description.
        newGroup = netPeerGroup.newGroup(groupID, implAdv, name,
            description);

        // Publier le groupe de manire distante seulement, comme newGroup()
        // se charge de le publier localement sur le pair.
        PeerGroupAdvertisement groupAdv = 
            newGroup.getPeerGroupAdvertisement();
        discovery.remotePublish(groupAdv, DiscoveryService.GROUP);
        
        // Dmarrer les applications du groupe de pair.
        newGroup.startApp(null);
    }

    /**
     * Dmarre la plateforme JXTA.
     *
     * @exception   PeerGroupException est lance si la plateforme ne peut tre
     *              dmarre.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        netPeerGroup = PeerGroupFactory.newNetPeerGroup();
    }

    /**
     * Dmarre l'application
     * @param args un tableau d'arguments de la ligne de commande
     */
    public static void main(String[] args)
    {
        NetSend netSend = new NetSend();
        
        try
        {           
            // Initialiser la plateforme JXTA
            netSend.initializeJXTA();

            // Crer le groupe
            netSend.createPeerGroup();
        }
        catch (Exception e)
        {
            System.out.println("Erreur au dmarrage de la plateforme JXTA : " + e);            
            System.exit(1);
        }
    }
}
